class Incident < ActiveRecord::Base
  include Filterable

  scope :has_number,      ->(number)    { where id: number.sub(/^#{INCIDENT_PREFIX}/, '').to_i }
  scope :created_since,   ->(datetime)  { where created_at: datetime.to_time..Time.now }
  scope :is_assigned_to,  ->(id)        { where assignee_id: id }
  scope :belongs_to_team, ->(id)        { where team_id: id }

  belongs_to :state
  belongs_to :impact
  belongs_to :priority
  belongs_to :urgency
  belongs_to :opening_category
  belongs_to :closing_category
  belongs_to :team
  belongs_to :requester, class_name: 'User', foreign_key: :requester_id
  belongs_to :agent, class_name: 'User', foreign_key: :agent_id
  belongs_to :assignee, class_name: 'User', foreign_key: :assignee_id
  has_many :logs

  has_many :affects
  has_many :cis, through: :affects

  has_many :causes
  has_many :causers, through: :causes
  has_many :inverse_causes, class_name: "Causes", foreign_key: "causer_id"
  has_many :inverse_causes, through: :inverse_causes, source: :incident

  validates :title, presence: true
  validates :description, presence: true
  validates :state_id, presence: true
  validates :impact_id, presence: true
  validates :priority_id, presence: true
  validates :urgency_id, presence: true
  validates :opening_category_id, presence: true
  validates :requester_id, presence: true
  validates :agent_id, presence: true
  validates :team_id, presence: true

  validate :assignee_part_of_team
  validate :state_in_progress
  validate :state_resolved
  validate :state_open

  private

  def assignee_part_of_team
    unless assignee_id.blank?
      team = Team.find(self.team_id)
      errors[:assignee_id] << 'não faz parte da equipe' unless team and team.users.include?(User.find(self.assignee_id))
    end
  end

  def state_open
    if state_id == State.open.id && opening_category_id == nil
      errors[:state_id] << 'não pode ser "Aberto" sem uma categoria'
    end
  end

  def state_in_progress
    if state_id == State.in_progress.id && assignee_id == nil
      errors[:state_id] << 'não pode ser "Em Progresso" sem um responsável'
    end
  end

  def state_resolved
    if state_id == State.resolved.id && closing_category_id == nil
      errors[:state_id] << 'não pode ser "Fechado" sem uma categoria de fechamento'
    end
  end
end
